TOM_BAGSPACE_VERSION = GetAddOnMetadata("TrackOMatic_BagSpace", "Version");
local PLUGIN_INTERFACE = 103;

local PLUGIN;

local PROF_BAG_IDS = {
                        22246, 30748, 67389, 22248, 22249, 21858, 41598, 54445,     -- enchanting bags
                        30745, 67390, 23774, 23775, 60217,                          -- engineering
                        30747, 67392, 24270, 70138,                                 -- gem bags
                        22250, 67393, 22251, 22252, 38225, 45773, 54446,            -- herb bags
                        39489, 67394, 44446, 70136,                                 -- inscription
                        67395, 34482, 34490, 38399,                                 -- leatherworking
                        30746, 67396, 29540, 38347, 70137,                          -- mining
                        60218,                                                      -- tackle boxes
};


function TrackOMatic_BagSpace_OnLoad(self)
    self:RegisterEvent("ADDON_LOADED");
end

local function registerPlugin()
    PLUGIN = TrackOMatic_GetPluginAPI("TrackOMatic_BagSpace", PLUGIN_INTERFACE);
    PLUGIN.name = "Bag Space";
    PLUGIN.version = TOM_BAGSPACE_VERSION;
    PLUGIN.author = GetAddOnMetadata("TrackOMatic_BagSpace", "Author");
    PLUGIN.category = "misc";
    PLUGIN.onUpdate = TrackOMatic_BagSpace_OnUpdate;
    PLUGIN:RegisterPlugin();
end

function TrackOMatic_BagSpace_OnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addon = ...;
        if (addon == "TrackOMatic_BagSpace") then
            registerPlugin();
        end
    end
end

local function isProfessionBag(bag)
    if ((bag >= 1) or (bag <= 4)) then
        local id = GetInventoryItemID("player", 19 + bag);
        if (id) then
            for i = 1, table.maxn(PROF_BAG_IDS), 1 do
                if (id == PROF_BAG_IDS[i]) then
                    return true;
                end
            end
        end
    end
    return false;
end

local function getTotalSlots(normalOnly)
    local slots = 0;
    for i = 0, 4, 1 do
        if ((not normalOnly) or (not isProfessionBag(i))) then
            slots = slots + GetContainerNumSlots(i);
        end
    end
    return slots;
end

local function getFreeSlots(normalOnly)
    local slots = 0;
    for i = 0, 4, 1 do
        if ((not normalOnly) or (not isProfessionBag(i))) then
            slots = slots + GetContainerNumFreeSlots(i);
        end
    end
    return slots;
end

local function switchMode(oldState)
    PLUGIN:SetData("includeProf", (not oldState));
    PLUGIN:UpdateTracker();
end

function TrackOMatic_BagSpace_OnUpdate()

    local data = PLUGIN:GetData();

    local value = getFreeSlots(not data.includeProf);
    local max = getTotalSlots(not data.includeProf);

    local tooltipTitle = "Free Bag Space";
    local tooltipText = value .. " of " .. max .. " slots free";
    if (not data.includeProf) then
        tooltipText = tooltipText .. "\n(Profession bags ignored)";
    end

    local pct = math.floor((value / max) * 100);
    local color = {r = (math.min(50, 100 - pct) / 50), g = (math.min(50, pct) / 50) * 0.75, b = 0};

    local menu = {
        { text = "Include profession bags", checked = data.includeProf, func = function() switchMode(data.includeProf); end },
    };

    local isGlowing = false;
    if (value <= 1) then
        isGlowing = true;
    end
    local barValue = value;
    if (value == 0) then
        barValue = max;
    end

    return barValue, max, color, "Bag Space", value .. "/" .. max .. " (" .. pct .. "%)", tooltipTitle, tooltipText, menu, isGlowing;

end
